/*
 * Copyright (c) 2006, team-naver.com
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN
 * NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.aibonware.viewnaver.config;

import javax.swing.*;
import org.w3c.dom.Element;

@SuppressWarnings("serial")
public class ThreadWindowConfig extends ConfigItem {
	public int maxThreadTabTitleLen = 10;
	public int left;
	public int top;
	public int width;
	public int height;

	public ThreadWindowConfig() {
		super("ThreadWindow");
		loadConfig(null);
	}

	@Override public void loadConfig(Element elem) {
		maxThreadTabTitleLen = Config.getIntConfig(elem, "MaxThreadTabTitleLen", 10);
		left = Config.getIntConfig(elem, "Left", 150);
		top = Config.getIntConfig(elem, "Top", 150);
		width = Config.getIntConfig(elem, "Width", 500);
		height = Config.getIntConfig(elem, "Height", 500);
	}

	@Override public void saveConfig(Element elem) {
		Config.setIntConfig(elem, "MaxThreadTabTitleLen", maxThreadTabTitleLen);
		Config.setIntConfig(elem, "Left", left);
		Config.setIntConfig(elem, "Top", top);
		Config.setIntConfig(elem, "Width", width);
		Config.setIntConfig(elem, "Height", height);
	}
	
	@Override public String getDispPath() {
		return "{.Xbh\";
	}
	
	private static class ThreadWindowConfigView extends ConfigItemView {
		private JTextField textField;
		private ThreadWindowConfig threadWindowConfig;
		
		ThreadWindowConfigView() {
			textField = new JTextField();

			addLine("^uő啶F\t", glue(textField));
		}

		public void setConfig(ConfigItem config) {
			threadWindowConfig = (ThreadWindowConfig)config;
			textField.setText(String.valueOf(threadWindowConfig.maxThreadTabTitleLen));
		}

		public void applyConfig() throws ConfigException {
			try {
				threadWindowConfig.maxThreadTabTitleLen = Integer.parseInt(textField.getText());
			} catch(NumberFormatException e) {
				throw new ConfigException("lw肵Ă");
			}
		}
	}

	@Override public ConfigItemView getView() {
		return new ThreadWindowConfigView();
	}
}
