/*
 * Copyright (c) 2006, team-naver.com
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN
 * NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.aibonware.viewnaver.model;

import java.util.*;

public class Board implements Iterable<NThread> {
	public final String boardName;
	public final String dispName;
	public final BoardType boardType;
	public final int pageNo;
	public final int totalPages;
	
	private Vector<NThread> threads = new Vector<NThread>();
	
	public Board(String boardName, String dispName, BoardType boardType, int pageNo, int totalPages) {
		this.boardName = boardName;
		this.dispName = dispName;
		this.boardType = boardType;
		this.pageNo = pageNo;
		this.totalPages = totalPages;
	}

	public void addTitle(NThread thread) {
		threads.addElement(thread);
	}

	public NThread getTitle(int index) {
		return (NThread)threads.elementAt(index);
	}
	
	public void removeTitle(int index) {
		threads.removeElementAt(index);
	}
	
	public int getThreadCount() {
		return threads.size();
	}

	public NThread getTitleByNid(int nid) {
		for(int i=0; i<getThreadCount(); i++) {
			NThread header = (NThread)getTitle(i);
			if(header.nid == nid) return header;	
		}
		
		throw new RuntimeException("invalid nid " + nid);
	}
	
	public Iterator<NThread> iterator() {
		return threads.iterator();
	}
	
	public String toString() {
		return dispName;
	}
}
