/*
 * Copyright (c) 2006, team-naver.com
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN
 * NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.aibonware.viewnaver.xml;

import java.text.*;
import java.util.*;
import org.w3c.dom.*;

import com.aibonware.viewnaver.model.*;

public class BaseXMLSerializer {
	protected Document doc;
    protected SimpleDateFormat df = new SimpleDateFormat("yyyy/MM/dd hh:mm:ss");
    
	public BaseXMLSerializer(Document doc) {
		this.doc = doc;
	}
	
	protected Element createAttrElem(String name, String value) {
		Element elem = doc.createElement(name);
		if(value == null) return elem;
		elem.appendChild(doc.createTextNode(value));
		
		return elem;
	}

	protected Element createAttrElem(String name, int value) {
		Element elem = doc.createElement(name);
		elem.appendChild(doc.createTextNode(String.valueOf(value)));
		
		return elem;
	}

	protected Element createAttrElem(String name, boolean value) {
		Element elem = doc.createElement(name);
		elem.appendChild(doc.createTextNode(String.valueOf(value)));
		
		return elem;
	}
	
	protected Element createAttrElem(String name, Date value) {
		Element elem = doc.createElement(name);
		if(value == null) return elem;
		elem.appendChild(doc.createTextNode(df.format(value)));

		return elem;
	}
   
	protected Element createAttrElem(String name, Country country) {
		Element elem = doc.createElement(name);
		elem.appendChild(doc.createTextNode(country.name()));

		return elem;
	}
}
